"use strict";
var intersection = require('lodash/intersection');
function buildResolver(spec) {
    var pool = null;
    function resolver(query) {
        var Model = pool.models[spec.name].sequelizeModel;
        var options = {};
        var attributes = pool.models[spec.name].sanitizeAttributes(query && query.attributes);
        if (attributes) {
            options.attributes = attributes;
        }
        var include = buildInclude(query, pool, spec.name);
        if (include && include.length > 0) {
            options.include = include;
        }
        if (query && (typeof query.byId === 'number' || typeof query.byId === 'string')) {
            return Model.findById(query.byId, options).then(function (data) { return ({ data: [data] }); });
        }
        var order = buildOrder(query, pool, spec.name);
        if (order) {
            options.order = order;
        }
        var where = buildWhere(query, pool, spec.name);
        if (where) {
            options.where = where;
        }
        var offset = query && query.pagination && query.pagination.skip;
        if (offset) {
            options.offset = offset;
        }
        var limit = query && query.pagination && query.pagination.take;
        if (limit) {
            options.limit = limit;
        }
        if (query && query.count) {
            return Model.findAndCount(options).then(function (_a) {
                var rows = _a.rows, count = _a.count;
                return ({ count: count, data: rows });
            });
        }
        return Model.findAll(options).then(function (data) { return ({ data: data }); });
    }
    function initialize(p) {
        pool = p;
    }
    return { resolver: resolver, initialize: initialize };
}
exports.buildResolver = buildResolver;
function buildOrder(query, pool, model) {
    var defaultOrder = pool.models[model].defaultOrder;
    var parameters = [];
    if (query && query.order && query.order.field && query.order.ordering !== 'NONE') {
        if (Array.isArray(query.order.chain)) {
            var currentModel_1 = model;
            query.order.chain.forEach(function (alias) {
                var nextModel = pool.models[currentModel_1].aliasModelMap[alias];
                parameters.push({ model: pool.models[nextModel].sequelizeModel, as: alias });
                currentModel_1 = nextModel;
            });
        }
        parameters.push(query.order.field);
        parameters.push(query.order.ordering);
    }
    if (parameters.length === 0) {
        if (!Array.isArray(defaultOrder)) {
            return null;
        }
        return [defaultOrder.slice()];
    }
    return [parameters];
}
function buildWhere(query, pool, model) {
    var where = {};
    if (query && typeof query.filters === 'object' && query.filters !== null) {
        var filtersKeys = Object.keys(query.filters);
        if (pool.models[model].allowedAttributes) {
            filtersKeys = intersection(filtersKeys, pool.models[model].allowedAttributes);
        }
        filtersKeys.map(function (field) {
            var value = query.filters[field];
            if (pool.models[model].fieldWhereBuilders &&
                typeof pool.models[model].fieldWhereBuilders[field] === 'function') {
                where[field] = pool.models[model].fieldWhereBuilders[field](value);
            }
            else if (value === null) {
                where[field] = null;
            }
            else {
                switch (typeof value) {
                    case 'array':
                        where[field] = { $in: value };
                        break;
                    case 'string':
                        where[field] = { $iLike: "%" + value + "%" };
                        break;
                    case 'boolean':
                    case 'number':
                        where[field] = value;
                        break;
                }
            }
        });
        return where;
    }
    return null;
}
function buildInclude(query, pool, model) {
    var queryInclude = query && query.include;
    var orderChain = query && query.order && query.order.chain;
    return buildNestedInclude(pool, queryInclude, orderChain, model);
}
function buildNestedInclude(pool, queryInclude, orderChain, model) {
    var aliasModelMap = pool.models[model].aliasModelMap;
    if (queryInclude || orderChain) {
        var include_1 = [];
        Object.keys(aliasModelMap).forEach(function (alias) {
            if (queryInclude[alias] || (orderChain && orderChain[0] === alias)) {
                var subQuery = queryInclude[alias];
                var targetModel = pool.models[aliasModelMap[alias]];
                var includedModel = {
                    model: targetModel.sequelizeModel,
                    as: alias
                };
                if (typeof subQuery === 'object' && subQuery) {
                    var attributes = targetModel.sanitizeAttributes(subQuery.attributes);
                    if (attributes) {
                        includedModel.attributes = attributes;
                    }
                    var where = buildWhere(subQuery, pool, aliasModelMap[alias]);
                    if (where) {
                        includedModel.where = where;
                    }
                }
                var nestedInclude = buildNestedInclude(pool, typeof subQuery === 'object' && subQuery.include, Array.isArray(orderChain) && orderChain.slice(1), aliasModelMap[alias]);
                if (nestedInclude) {
                    includedModel.include = nestedInclude;
                }
                include_1.push(includedModel);
            }
        });
        return include_1;
    }
    return null;
}
